/*
* Questo sketch utilizza Arduino Mega 2560 e display TFT ILI9327, con libreria MCUFRIEND, un RTC DS3231 e un sensore
* BME280.
* Visualizza ora, minuti, giorno, data, oltre a temperatura, umidità e pressione.
* Per poter utilizzare il display ILI9327, ho dovuto fare delle modifiche alla libreria MCUFRIEND.cpp, 
* alle righe 16, 25 e 28.
*
* Ho aggiunto anche il refresh del display ogni 240 secondi, utilizzando TimedAction.
* 
* Il 4 marzo 2023 ho fatto modifiche per visualizzare i valori Min e Max, con i decimali.
*
* Infine, ho aggiunto la formula di conversione del valore di pressione, per ottenere 
* un valore corrispondente alla pressione a livello  del mare (è necessario indicare 
* l'altezza in metri sul livello del mare, alla riga 58). 
*
* Il 8 marzo 2023 ho fatto modifiche "estetiche", ripristinando i due punti tra ore e
* minuti, aumentando la dimensione dei valori Min e Max.
*
* Il 9 marzo ho aggiunto il soft reset alle ore 00.01.01 di tutti i giorni. Verificare!!
* 
* Il 10 Marzo ho aggiunto le istruzioni per scrivere sulla scheda micro SD la data e i 
* valori * Min e Max di Temperatura, Umidità e Pressione alle ore 23:59:01 di ogni giorno.
* L'11 Marzo ho aggiunto la visualizzazione sul display dell'avvenuta o mancata 
* inizializzazione della micro SD (Solo dopo il reset, e fino al successivo refresh del 
* display, in alto a sinistra del display appare "SD", se inizializzata, oppure "SD!", 
* se  non inizializzata) e dell'avvenuta o mancata scrittura dei dati sulla SD (dopo le 
* ore 23.59.01, appare "MEM", se i dati sono stati scritti oppure "NO MEM" in caso contrario,
* ma resta visibile solo fino al successivo reset). 
*
* Aggiunto circuito divisore di tensione (D3-----R4.7K--|--R10K----GND) e la riga "pinMode(3, OUTPUT);" a setup
*                                                       |
*                                                       D3
* 
* per far spegnere la retroilluminazione del display tra mezzanotte e le 7.30, facendo modifiche
* anche allo sketch.
* 
* Il 15 Marzo ho aggiunto un pulsante, collegato tra PIN 2 e massa, e una resistenza tra PIN 2 e +5V,
* per poter accendere la retroilluminazione quando si spegne tra mezzanotte e le 7, facendo anche le
* modifiche allo sketch.
*
* Ad ogni reset o ad ogni accensione del dispositivo, si inizializza la micro SD e appare SD sul display.
* Alle 23:59:01, scrive su micro SD la data, e i Min e Max della giornata, e appare MEM sul display.
* Alle 00:01:01 resetta Arduino.
* Alle 00:06:00 spegne la retroilluminazione del display (che può essere accesa momentaneamente,
* premendo il pulsante posto sopra la presa USB).
* 
* Caricato su Instructables.com.
* 
* Caricato su Arduino Mega il 4/3/2023
*
* © P.L. 03/2023 - Giano2002
*/

#include <BME280I2C.h>
#include <MCUFRIEND_kbv.h>
#include <RTClib.h>
#include <TimedAction.h>
#include <SPI.h>
#include <SD.h>

RTC_DS1307 rtc;
MCUFRIEND_kbv tft;
BME280I2C bme;
File myFile;

char daysOfTheWeek[7][12] = { "Domenica", " Lunedi", " Martedi", "Mercoledi", " Giovedi", " Venerdi", " Sabato" };
char month[12][12] = { "Dic", "Gen", "Feb", "Mar", "Apr", "Mag", "Giu", "Lug", "Ago", "Set", "Ott", "Nov" };
uint16_t ID, currentcolor;
uint8_t Orientation = 1;  //Orientamento display

float tmin = 100;
float tmax = -100;
float umin = 100;
float umax = 0;
float pmin = 2000;
float pmax = 0;
float temp, hum, pres;
float pres1;
float altitudepws = 29.60;  // altitudine slm di casa a Roma = 30 m. - Modificare il valore per altitudini diverse

String data = "";

///// Assegnare un nome al codice dei colori //////
#define BLACK 0x0000
#define BLUE 0x001F
#define RED 0xF800
#define GREEN 0x07E0
#define CYAN 0x07FF
#define MAGENTA 0xF81F
#define YELLOW 0xFFE0
#define WHITE 0xFFFF
#define ORANGE 0xFDA0
#define NAVY 0x000F
#define GREENYELLOW 0xB7E0
#define OLIVE 0x7BE0
#define LIGHTGREY 0xC618
#define DARKGREY 0x7BEF
#define PINK 0xFC9F
#define PURPLE 0x780F

//////////// Refresh del display //////////////////////////
void (*resetFunc)(void) = 0;  // dichiarare la funzione di reset all'indirizzo 0.

void tftrefresh();
TimedAction timedAction = TimedAction(240000, tftrefresh);  // Intervallo di 240 secondi per il refresh del display. Questo tempo può essere modificato.

///////////////////////////////
void setup() {
  Serial.begin(9600);
  pinMode(3, OUTPUT);  // PIN D3 come uscita
  pinMode(2, INPUT);  // PIN D2 come input 
  digitalWrite(3, HIGH);
  Wire.begin();
  bme.begin();
  while (!Serial)
    ;
  {
    delay(100);
  }
  Serial.println("BME280 test!");

  if (!bme.begin()) {
    Serial.print("Non trovo il sensore BME!");
    while (true)
      ;
  }
  if (!rtc.begin()) {

    Serial.println("Non trovo RTC");
    while (1)
      ;
  }
  if (!rtc.isrunning()) {
    Serial.println("RTC non sta funzionando!");
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }
  /////////////////////////////////////////////////////////////////////
  //rtc.adjust(DateTime(2023, 3, 16, 9, 13, 0)); /////////////////////
  /////////////////////////////////////////////////////////////////////
  //  Per modificare data e ora, occorre agire sulla riga sopra, utilizzando il seguente formato: Anno (4 cifre),
  //  mese, giorno, ora, minuti, secondi. Quindi, togliere il commento e caricare lo sketch. Dopo di che, rimettere il
  //  commento e ricaricare di nuovo lo sketch.

  uint16_t tmp;
  tft.reset();
  ID = tft.readID();
  tft.begin(ID);
  tft.setRotation(Orientation);
  tft.fillScreen(NAVY);
  uint16_t ID = tft.readID();
  Serial.print("Sto inizializzando la micro SD...");

  if (!SD.begin(4)) {
    Serial.println("inizializzazione fallita!");
    tft.setCursor(10, 7);
    tft.setTextColor(RED, NAVY);
    tft.setTextSize(2);
    tft.print("SD!");

    while (1)
      ;
  }
  Serial.println("inizializzazione riuscita.");

  tft.setCursor(10, 7);
  tft.setTextColor(WHITE, NAVY);
  tft.setTextSize(2);
  tft.print("SD");
}
 /////////////////////////////////////////////////////////////////////////
void loop() {

  DateTime now = rtc.now();
  bme.read(pres, temp, hum);
  pres1 = (pres / pow(1 - (altitudepws / 44330), 5.255));  // Formula per convertire da pressionee rel. a pressione slm.

  ///////////////// Scrivere ANNO e mese nell'angolo superiore destro del display/////////////////////////
  tft.setCursor(380, 0);
  tft.setTextSize(1);
  tft.setTextColor(DARKGREY, NAVY);
  tft.print("02");
  tft.setCursor(386, 9);
  tft.print("0");
  tft.setCursor(386, 18);
  tft.print("2");
  tft.setCursor(386, 26);
  tft.print("3");

  /////////////// Disegnare 2 cerchi e, al loro interno, le iniziali del mio nome //////

  for (int i = 15; i > 3; i -= 3) {
    tft.drawCircle(69, 38, i, ORANGE);
    tft.setCursor(67, 36);
    tft.setTextSize(1);
    tft.setTextColor(WHITE, NAVY);
    tft.print("P");
    delay(100);
  }
  for (int i = 15; i > 3; i -= 3) {
    tft.drawCircle(339, 40, i, ORANGE);
    tft.setCursor(338, 36);
    tft.print("L");
    delay(100);
  }

  //////////////Disegnare le linee ////////////////////////////////////
  tft.drawLine(0, 203, 400, 203, WHITE);    // riga inferiore orizzontale
  tft.drawLine(200, 100, 200, 200, WHITE);  // riga centrale verticale
  tft.drawLine(0, 100, 400, 100, WHITE);    // riga superiore orizzontale
  tft.drawLine(0, 235, 400, 235, YELLOW);   // ULTIMA riga inferiore orizzontale

  ////// Scrivere "Ore" e "Minuti" /////////

  tft.setTextColor(ORANGE, NAVY);
  tft.setCursor(150, 0);
  tft.setTextSize(1);
  tft.print("Ore");
  tft.setCursor(245, 0);
  tft.print("Minuti");
  tft.setTextColor(RED, NAVY);
  tft.setTextSize(6);
  tft.setCursor(113, 18);
  if (now.hour() <= 9) {
    tft.print("0");
    tft.print(now.hour(), DEC);
  } else {
    tft.print(now.hour(), DEC);
  }

  ////// separatore tra ore e minuti (:) //////////////
  tft.setCursor(190, 18);
  tft.print(":");

  ////////////////////////////////////////////////

  tft.setTextColor(RED, NAVY);
  tft.setCursor(226, 20);
  if (now.minute() <= 9) {
    tft.print("0");
    tft.print(now.minute(), DEC);
  } else {
    tft.print(now.minute(), DEC);
  }

  Serial.print(now.second());
  //////////////Scrivere la data ///////////////////////////////////

  tft.reset();
  tft.setCursor(9, 70);  // 9= distanza dal bordo sinistro; 70= distanza dal bordo superiore del display.
  tft.setTextSize(3);
  tft.setTextColor(GREEN, NAVY);
  tft.print(String(daysOfTheWeek[now.dayOfTheWeek()]));
  tft.print(" ");
  tft.print(now.day(), DEC);
  tft.print(" ");
  tft.print(String(month[now.month()]));
  tft.print(" ");
  tft.print(now.year(), DEC);

  ///// Visualizzare i valori di Temperatura, Umidità e Pressione sul display ////////////
  tft.setTextColor(PINK, NAVY);
  tft.setTextSize(2);
  tft.setCursor(20, 105);
  tft.print("Temperatura");
  tft.print(" C");
  tft.setCursor(250, 105);
  tft.setTextColor(CYAN, NAVY);
  tft.print("Umidita R.");
  tft.setTextSize(5);
  tft.setCursor(30, 132);
  tft.setTextColor(YELLOW, NAVY);
  tft.print(data = String(temp, 1));
  tft.setTextSize(1);
  tft.print(" O");
  tft.setTextSize(5);
  tft.setTextColor(CYAN, NAVY);
  tft.setCursor(235, 132);
  tft.print(data = String(hum, 1));
  tft.setTextSize(3);
  tft.setCursor(355, 137);
  tft.print(" %");
  //////////////// Rilevare i valori Min e Max e visualizzarli /////////////////

  if (temp < tmin) {
    tmin = temp;
  }
  if (temp > tmax) {
    tmax = temp;
  }
  if (hum < umin) {
    umin = hum;
  }
  if (hum > umax) {
    umax = hum;
  }
  if (pres1 < pmin) {
    pmin = pres1;
  }
  if (pres1 > pmax) {
    pmax = pres1;
  }

  tft.setCursor(0, 182);
  tft.setTextColor(WHITE, NAVY);
  tft.setTextSize(2);
  tft.print("Min ");
  tft.print(data = String(tmin, 1));
  tft.setCursor(100, 182);
  tft.setTextColor(RED, NAVY);
  tft.print("Max ");
  tft.print(data = String(tmax, 1));
  tft.setCursor(202, 182);
  tft.setTextColor(WHITE, NAVY);
  tft.setTextSize(2);
  tft.print("Min ");
  tft.print(data = String(umin, 1));
  tft.setCursor(302, 182);
  tft.setTextColor(RED, NAVY);
  tft.print("Max ");
  tft.print(data = String(umax, 1));

  //////// Visualizzare i valori di Pressione ///////////////////////
  tft.setTextSize(2);
  tft.setCursor(5, 213);
  tft.setTextColor(GREEN, NAVY);
  tft.print(data = String(pres1, 1));
  tft.print(" hPa|");
  tft.setTextColor(WHITE, NAVY);
  tft.print("Min ");
  tft.print(data = String(pmin, 1));
  tft.setCursor(263, 213);
  tft.setTextColor(RED, NAVY);
  tft.print("|Max ");
  tft.print(data = String(pmax, 1));

  //////////////////// Scrivere su micro SD tutti i valori Min, MAx e la data, ogni mezzanotte///////
  // Ho dovuto usare <now.second() ==1 || now.swcond() ==2> per tener conto che il conteggio dei secondi è irregolare.

  if ((now.hour() == 23) && (now.minute() == 59) && (now.second() == 1 || (now.second() == 2))) {
    myFile = SD.open("test.txt", FILE_WRITE);
    // se il file si apre...
    if (myFile) {
      Serial.print("Scrivo su test.txt...");
      myFile.println(now.day(), DEC);
      myFile.println(String(month[now.month()]));
      myFile.println(now.year(), DEC);
      myFile.println(data = String(tmin, 1) + " T min");
      myFile.println(data = String(tmax, 1) + " T max");
      myFile.println(data = String(umin, 1) + " U min");
      myFile.println(data = String(umax, 1) + " U max");
      myFile.println(data = String(pmin, 1) + " P min");
      myFile.println(data = String(pmax, 1) + " P max");
      // chiudere il file:
      myFile.close();
      Serial.println("Fatto!.");
      tft.setCursor(10, 7);
      tft.setTextColor(WHITE, NAVY);
      tft.setTextSize(2);
      tft.print("Mem");  // Scrittura su SD effettuata.

    } else {
      // Se il file non si apre, scrivere "errore":
      Serial.println("Errore di apertura del test.txt");
      tft.setCursor(10, 7);
      tft.setTextColor(RED, NAVY);
      tft.setTextSize(2);
      tft.print("No mem!");  // Scrittura su SD non effettuata.
    }
    ///////////////////////////////////////////////////////////
  }

  ///// Resettare Arduino alla mezzanotte e un minuto /////////////////
  // Ho dovuto usare <now.second() ==1 || now.second() ==2> per tener conto che il conteggio dei secondi è irregolare.

  if ((now.hour() == 0) && (now.minute() == 1) && (now.second() == 1 || (now.second() == 2))) resetFunc();
  /////////////////////////////////////////////////////////////
 
  /////Pulire il dipslay, utilizzando il tempo impostato alla riga <TimedAction timedAction = TimedAction(240000, tftrefresh);>

  ///// Spegnere la retroilluminazione del display tra le ore 0:5 e le ore 7  ///////////////
  if ((now.hour() >= 7) || ((now.hour() <= 0 && (now.minute() <= 5)))) {
    digitalWrite(3, HIGH);
  } else {
    digitalWrite(3, LOW);
  }

  ///////////// accendere il display momentaneamente, usando il pulsante ///////////////
  
  if (digitalRead(2) == LOW) { // Se si preme il pulsante
    digitalWrite(3, HIGH); // Porta Alto il PIN 3 e quindi accende la retroilluminazione del display.
  }

  //////////////////////////////////////
  /////Pulire il dipslay, utilizzando il tempo impostato alla riga <TimedAction timedAction = TimedAction(240000, tftrefresh);>
  
  timedAction.check();
}
void tftrefresh() {
  tft.setCursor(0, 0);
  tft.setTextSize(2);
  tft.setTextColor(WHITE, BLACK);
  tft.print("*****************************************************************************************************");
  tft.print("*****************************************************************************************************");
  tft.print("*****************************************************************************************************");
  tft.print("*****************************************************************************************************");
  tft.print("*****************************************************************************************************");
  tft.setCursor(0, 0);
  tft.setTextColor(BLACK, WHITE);
  tft.print("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&");
  tft.print("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&");
  tft.print("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&");
  tft.print("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&");
  tft.print("&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&");
  delay(300);
  tft.fillScreen(BLACK);
  delay(200);
  tft.fillScreen(NAVY);
  delay(200);
}
